@@ -164,7 +164,7 @@ def load_tf_generator():
         return None
 
 def pip_install_tflite():
-    """Makes sure TFLite is installed
+    """Makes sure TFLite is installed by executing a pip install command from the command line (sub-optimal solution)
 
     Parameters: 
         none
@@ -237,8 +237,53 @@ def load_csv_data(csv_file='data_test.csv'):
 
     return df
 
+def load_exercise_threshold_app_data(data_dict={}):
+    """Loads data from data dict with format provided by https://www.exercisethresholds.com/
+
+    Parameters:
+        data_dict (dict) : Dictionary with format like test/exercise_threshold_app_test.json
+
+    Returns:
+        df (pandas df) : Pandas data frame with format that can be used by Pyoxynet for inference (columns needed: 'VO2_I', 'VCO2_I', 'VE_I', 'PetO2_I', 'PetCO2_I', 'VEVO2_I', 'VEVCO2_I')
+
+    """
+
+    import json
+    import pandas as pd
+
+    time = []
+    VO2_I = []
+    VCO2_I = []
+    VE_I = []
+    PetO2_I = []
+    PetCO2_I = []
+    VEVO2_I = []
+    VEVCO2_I = []
+
+    for data_points_ in data_dict[0]['data']:
+        time.append(data_points_['t'])
+        VO2_I.append(data_points_['VO2'])
+        VCO2_I.append(data_points_['VCO2'])
+        VE_I.append(data_points_['VE'])
+        PetO2_I.append(data_points_['PetO2'])
+        PetCO2_I.append(data_points_['PetCO2'])
+        VEVO2_I.append(data_points_['VE/VO2'])
+        VEVCO2_I.append(data_points_['VE/VCO2'])
+
+    df = pd.DataFrame()
+    df['time'] = [time_ - time[0 ]for time_ in time]
+    df['VO2_I'] = VO2_I
+    df['VCO2_I'] = VCO2_I
+    df['VE_I'] = VE_I
+    df['PetO2_I'] = PetO2_I
+    df['PetCO2_I'] = PetCO2_I
+    df['VEVO2_I'] = VEVO2_I
+    df['VEVCO2_I'] = VEVCO2_I
+
+    return df
+
 def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
-    """Test if the pyoxynet pipeline is running correclty
+    """Runs the pyoxynet inference
 
     Parameters: 
         n_inputs (int) : Number of inputs (deafult to Oxynet configuration)
@@ -251,13 +296,15 @@ def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
 
     import numpy as np
     from uniplot import plot
+    import pandas as pd
+    from scipy import stats
 
     import json
 
     tfl_model = load_tf_model(n_inputs=n_inputs, past_points=past_points)
 
     if len(input_df) == 0:
-        print('Using default py-oxynet data')
+        print('Using default pyoxynet data')
         df = load_csv_data()
     else:
         df = input_df
@@ -273,6 +320,18 @@ def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
 
     # df = pd.DataFrame.from_dict(data)
 
+    # some adjustments to input df
+    # TODO: create dedicated function for this
+    df = df.drop_duplicates('time')
+    df['timestamp'] = pd.to_datetime(df['time'], unit='s')
+    df = df.set_index('timestamp')
+    df = df.resample('1S').mean()
+    df = df.interpolate()
+    df['VO2_20s'] = df.VO2_I.rolling(20, win_type='triang', center=True).mean().fillna(method='bfill').fillna(
+        method='ffill')
+    df = df.reset_index()
+    df = df.drop('timestamp', axis=1)
+
     if n_inputs==7 and past_points==40:
         X = df[['VO2_I', 'VCO2_I', 'VE_I', 'PetO2_I', 'PetCO2_I', 'VEVO2_I', 'VEVCO2_I']]
     if n_inputs==5 and past_points==40:
@@ -290,8 +349,8 @@ def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
     p_3 = []
     time = []
 
-    for i in np.arange(len(XN)-time_series_len):
-        XN_array = np.asarray(XN[i:(i+time_series_len)])
+    for i in np.arange(time_series_len, len(XN)):
+        XN_array = np.asarray(XN[(i-time_series_len):i])
         input_data = np.reshape(XN_array, input_details[0]['shape'])
         input_data = input_data.astype(np.float32)
         tfl_model.allocate_tensors()
@@ -303,12 +362,23 @@ def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
         p_3.append(output_data[0][2])
         time.append(df.time[i])
 
-    import pandas as pd
-    df = pd.DataFrame()
-    df['time'] = time
-    df['p_md'] = p_1
-    df['p_hv'] = p_2
-    df['p_sv'] = p_3
+    tmp_df = pd.DataFrame()
+    tmp_df['time'] = time
+    tmp_df['p_md'] = p_1
+    tmp_df['p_hv'] = p_2
+    tmp_df['p_sv'] = p_3
+
+    mod_col = tmp_df[['p_md', 'p_hv', 'p_sv']].iloc[:20].mean().idxmax()
+    sev_col = tmp_df[['p_md', 'p_hv', 'p_sv']].iloc[-20:].mean().idxmax()
+    for labels_ in ['p_md', 'p_hv', 'p_sv']:
+        if labels_ not in [mod_col, sev_col]:
+            hv_col = labels_
+
+    out_df = pd.DataFrame()
+    out_df['time'] = time
+    out_df['p_md'] = tmp_df[mod_col]
+    out_df['p_hv'] = tmp_df[hv_col]
+    out_df['p_sv'] = tmp_df[sev_col]
 
     plot([p_1, p_2, p_3],
          title="Exercise intensity domains",
@@ -316,9 +386,38 @@ def test_pyoxynet(input_df=[], n_inputs=7, past_points=40):
          color=True,
          legend_labels=['1', '2', '3'])
 
-    return df
+    out_dict = {}
+    out_dict['VT1'] = {}
+    out_dict['VT2'] = {}
+    out_dict['VT1']['time'] = {}
+    out_dict['VT2']['time'] = {}
+
+    VT1_index = int(out_df[(out_df['p_hv'] <= out_df['p_md']) & (out_df['p_hv'] > 0.1)].index[-1] + time_series_len)
+    VT2_index = int(out_df[(out_df['p_sv'] <= out_df['p_hv']) & (out_df['p_sv'] > 0.1)].index[-1] + time_series_len)
+
+    out_dict['VT1']['time'] = df.iloc[VT1_index]['time']
+    out_dict['VT2']['time'] = df.iloc[VT2_index]['time']
+
+    out_dict['VT1']['VO2'] = df.iloc[VT1_index]['VO2_20s']
+    out_dict['VT2']['VO2'] = df.iloc[VT2_index]['VO2_20s']
+
+    return out_df, out_dict
 
 def create_probabilities(duration=600, VT1=320, VT2=460):
+    """Creates the probabilities of being in different intensity domains
+
+    These probabilities are then sent to the CPET generator and they are used ot generate CPET vars that can replicate those probabilities
+
+    Parameters:
+        length (int): Length of the output list
+        scale_factor (float): Scale factor to be applied to the whole output
+        variation (float): Local variation of the main signal with the random walk
+
+    Returns:
+        none
+
+    """
+
     import numpy as np
 
     time_length = np.arange(1, duration + 1)
@@ -333,6 +432,18 @@ def create_probabilities(duration=600, VT1=320, VT2=460):
     return p_mF, p_hF, p_sF
 
 def random_walk(length=1, scale_factor=1, variation=1):
+    """Random walk generator
+
+    Parameters:
+        length (int): Length of the output list
+        scale_factor (float): Scale factor to be applied to the whole output
+        variation (float): Local variation of the main signal with the random walk
+
+    Returns:
+        none
+
+    """
+
     from random import seed
     from random import random
 
@@ -346,6 +457,18 @@ def random_walk(length=1, scale_factor=1, variation=1):
     return [i/scale_factor for i in random_walk]
 
 def generate_CPET(generator, plot=False, fitness_group=None):
+    """Actually generates the CPET file
+
+    Parameters:
+        length (int): Length of the output list
+        scale_factor (float): Scale factor to be applied to the whole output
+        variation (float): Local variation of the main signal with the random walk
+
+    Returns:
+        df (pd df): Pandas dataframe with CPET data included and ready to be processed by the model (if needed)
+        data (dict): Data realative to the generated CPET
+
+    """
 
     import random
     import numpy as np
